#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cli = void 0;
const child_process_1 = require("child_process");
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
// console colors
const FgRed = "\x1b[31m";
const FgGreen = "\x1b[32m";
const FgBlue = "\x1b[34m";
const FgReset = "\x1b[0m";
/**
 * execSync with standard input outputs
 */
function execSyncWithIo(cmd) {
    return (0, child_process_1.execSync)(cmd, { stdio: "inherit" });
}
/**
 * Init workspace
 * install pnpm globally
 * create pnpm workspace
 * create workspace package.json
 * @param options options
 * @param options.dry if true, init will be simulated but not executed
 * @param options.update if true, init will be done even if config files are already present
 * @param options.pnpmVersion if given, fix version of pnpm
 */
function initWorkspace(options) {
    // check if user is on Windows & current path contains spaces
    const operatingSystem = process.platform;
    const currentPath = process.cwd();
    if (operatingSystem === "win32" && /\s/.test(currentPath)) {
        console.error(FgRed + "Your workspace path should not contain any whitespaces when using Windows", FgReset);
        process.exit(4 /* ErrorCodes.WRONG_PATH_FORMAT */);
    }
    // check Node Version
    let nodeVersionStr = process.version.toLowerCase();
    if (nodeVersionStr.startsWith("v")) {
        nodeVersionStr = nodeVersionStr.substring(1);
    }
    const nodeMainVersion = Number(nodeVersionStr.substring(0, nodeVersionStr.indexOf(".")));
    console.info("* Node version", nodeMainVersion, "(" + process.version + ")");
    if (nodeMainVersion < 14) {
        console.error(FgRed + "Node version 14 or above is required, init aborted", FgReset);
        console.info("1. Install NVM");
        console.info(" Linux: https://github.com/nvm-sh/nvm");
        console.info(" Windows: https://github.com/coreybutler/nvm-windows");
        console.info("2. Install Node");
        console.info(" nvm install 18.17.0");
        console.info(" nvm use 18.17.0");
        process.exit(1 /* ErrorCodes.WRONG_NODE */);
    }
    // check config files are not already in current folder
    const configFiles = ["package.json", "pnpm-workspace.yaml"];
    {
        for (const cf of configFiles) {
            if (fs.existsSync("./" + cf)) {
                if (!(options === null || options === void 0 ? void 0 : options.update)) {
                    console.error(FgRed + cf + " found - init cancelled", FgReset);
                    process.exit(2 /* ErrorCodes.INIT_BUT_FILE_EXISTS */);
                }
            }
            else if (options === null || options === void 0 ? void 0 : options.update) {
                console.error(FgRed + cf + " was not found");
                console.error(FgRed + "- Are you at the workspace root ?", FgReset);
                console.error(FgRed + "- Do you want to initialize a new workspace ? then you need to remove --update", FgReset);
                process.exit(3 /* ErrorCodes.UPDATE_BUT_FILE_DOES_NOT_EXIST */);
            }
        }
    }
    // Install pnpm
    const pnpmFixedVersion = options.pnpmVersion ? "@" + options.pnpmVersion : "";
    if (!(options === null || options === void 0 ? void 0 : options.update)) {
        console.info("* install pnpm : npm install -g pnpm" + pnpmFixedVersion);
        if (!(options === null || options === void 0 ? void 0 : options.dry)) {
            execSyncWithIo("npm install -g pnpm" + pnpmFixedVersion);
        }
    }
    else {
        console.info("* update pnpm : pnpm add -g pnpm" + pnpmFixedVersion);
        if (!(options === null || options === void 0 ? void 0 : options.dry)) {
            execSyncWithIo("pnpm add -g pnpm" + pnpmFixedVersion);
        }
    }
    console.info("* display pnpm version : pnpm -v");
    execSyncWithIo("pnpm -v");
    // install lpx globally
    console.info("* " + (!(options === null || options === void 0 ? void 0 : options.update) ? "install" : "update") + " lpx globally : pnpm add -g lpx");
    if (!(options === null || options === void 0 ? void 0 : options.dry)) {
        execSyncWithIo("pnpm add -g lpx");
    }
    // create configuration files
    const from = path.resolve(__dirname, "../workspace") + "/_";
    for (const cf of configFiles) {
        console.info("* copy", path.resolve(from + cf), " => ", path.resolve("./" + cf));
        if (!(options === null || options === void 0 ? void 0 : options.dry)) {
            fs.copyFileSync(from + cf, "./" + cf);
        }
    }
    // create nvmrc
    if (!fs.existsSync("./.nvmrc")) {
        console.info("*", process.version, "> .nvmrc");
        if (!(options === null || options === void 0 ? void 0 : options.dry)) {
            fs.writeFileSync("./.nvmrc", process.version);
        }
    }
    if (!(options === null || options === void 0 ? void 0 : options.update)) {
        // install packages
        console.info("* init done, you can now checkout cervval.js and install packages");
        console.info("git clone https://git.cervval.com/CERVVAL/cervval.js.git");
        console.info("pnpm install");
    }
    else {
        console.info(FgBlue + "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
        console.info(FgBlue + "*update done, you can now install packages");
        console.info(FgGreen + "pnpm install", FgReset);
    }
    return true;
}
/**
 * parses command arguments and inits workspace in current working directory
 */
function cli() {
    const options = { dry: false, update: false, pnpmVersion: "8.14.1" };
    const args = process.argv.slice(2);
    if (args.includes("--dry")) {
        console.info("* dry mode on");
        options.dry = true;
    }
    if (args.includes("--update")) {
        console.info("* update mode on");
        options.update = true;
    }
    try {
        initWorkspace(options);
    }
    catch (err) {
        console.error(err);
        process.exit(99 /* ErrorCodes.UNEXPECTED */);
    }
}
exports.cli = cli;
cli();
//# sourceMappingURL=index.js.map